/*
 * @(#)Component.java	0.002 1999/11/15
 *
 * Copyright (c) 1999- by Daichi T.GOTO. All Rights Reserved.
 *
 */
package yuzu;

import java.io.*;
import java.util.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;

import yuzu.tree.*;
import yuzu.util.*;
import yuzu.table.*;
import yuzu.disp.*;
import yuzu.mail.get.*;
import yuzu.mail.send.*;
import yuzu.addressbook.*;


class Components extends JFrame {

	protected static Container container = null;
	protected static JFrame frame = null;

	protected static JMenuBar menubar = new JMenuBar();
	protected static JMenu menu_file = new JMenu();
	protected static JMenu menu_edit = new JMenu();
	protected static JMenu menu_set  = new JMenu();
	protected static JMenu menu_info = new JMenu();

	protected static JMenuItem menu_file_getmail = new JMenuItem();
	protected static JMenuItem menu_file_sendmail = new JMenuItem();
	protected static JMenuItem menu_file_quit = new JMenuItem();

	protected static JMenuItem menu_edit_copy     = new JMenuItem();
	protected static JMenuItem menu_edit_paste    = new JMenuItem();
	protected static JMenuItem menu_edit_allcopy  = new JMenuItem();
	protected static JMenuItem menu_edit_cut      = new JMenuItem();

	protected static JMenu menu_set_lang  = new JMenu();
	protected static ButtonGroup groupe_langmenu = new ButtonGroup();
	protected static JRadioButtonMenuItem menu_set_lang_ja = new JRadioButtonMenuItem();
	protected static JRadioButtonMenuItem menu_set_lang_en = new JRadioButtonMenuItem();

	protected static JMenuItem menu_info_version = new JMenuItem();

	protected static final String sp = System.getProperty("file.separator");
 	protected static ImageIcon yuzumark     = new ImageIcon("images"+sp+"YuzuImage.jpeg");
 	protected static ImageIcon mailgetIcon  = new ImageIcon("images"+sp+"mailget.gif");
 	protected static ImageIcon mailsendIcon = new ImageIcon("images"+sp+"mailsend.gif");
 	protected static ImageIcon quitIcon     = new ImageIcon("images"+sp+"quit.gif");
	protected static ImageIcon copyIcon     = new ImageIcon("images"+sp+"copy.gif");
	protected static ImageIcon allcopyIcon  = new ImageIcon("images"+sp+"allcopy.gif");
	protected static ImageIcon pasteIcon    = new ImageIcon("images"+sp+"paste.gif");
	protected static ImageIcon cutIcon      = new ImageIcon("images"+sp+"cut.gif");
 	protected static ImageIcon langIcon     = new ImageIcon("images"+sp+"lang.gif");
 	protected static ImageIcon verIcon      = new ImageIcon("images"+sp+"version.gif");
	protected static ImageIcon Icon         = new ImageIcon("images"+sp+"icon.gif");

	protected static JPanel pane            = new JPanel(new BorderLayout());
	protected static JPanel treepane        = new JPanel(new BorderLayout());
	protected static DTree tree             = new DTree();
	protected static JPanel addresspane     = new JPanel(new BorderLayout());
	protected static Addressbook address    = new Addressbook();
	protected static JPanel tablepane       = new JPanel(new BorderLayout());
	protected static DTable table           = new DTable();
	protected static JPanel textpane        = new JPanel(new BorderLayout());
	protected static Disp disp              = new Disp();
	protected static JSplitPane splitHor    = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT);
	protected static JSplitPane splitRightVer = new JSplitPane(JSplitPane.VERTICAL_SPLIT);
	protected static JSplitPane splitLeftVer  = new JSplitPane(JSplitPane.VERTICAL_SPLIT);

	private static ResourceBundle resource = 
		ResourceBundle.getBundle(sp+"lib"+sp+"resource", Locale.getDefault());

 	protected static DMailgetDialog  mailgetdialog  = new DMailgetDialog();
	protected static DMailsendDialog mailsenddialog = new DMailsendDialog();

	protected static Thread mailgetthread = null;
	protected static Thread mailsendthread = null;

	protected static Utilities util = new Utilities();
	protected static StatusHoldObject statusholdobject = null;


	Components() {
		this.addWindowListener(new Closing());
		try {
			UIManager.setLookAndFeel("javax.swing.plaf.metal.MetalLookAndFeel");
			SwingUtilities.updateComponentTreeUI(this);
		}
		catch(UnsupportedLookAndFeelException even) {}
		catch(ClassNotFoundException even) {}
		catch(InstantiationException even) {}
		catch(IllegalAccessException even) {}
		container = super.getContentPane();
		container.setLayout(new BorderLayout());
		frame = this;
		
		statusholdobject = util.getStatusHoldObject();
	}

	protected void refresh() {
		SwingUtilities.invokeLater(new Runnable() {
		  public void run() {
 			  pane.setVisible(false);
 			  pane.setVisible(true);
		  }
		});
	}

	protected class Closing extends WindowAdapter implements ActionListener {
		public void windowClosing(WindowEvent even) {
			Terminator();
		}
		public void actionPerformed(ActionEvent even) {
			Terminator();
		}
	}

	protected void Terminator() {
		disp.Terminator();

		statusholdobject.setWindowSizeX(frame.getSize().width);
		statusholdobject.setWindowSizeY(frame.getSize().height);
		statusholdobject.save();
		File workDir = util.getWorkDir();
		String[] tempfiles = workDir.list();
		for (int count=0; count<tempfiles.length; count++) {
			(new File(workDir, tempfiles[count])).delete();
		}
		System.exit(0);
	}

	public void print(String string) {
		System.out.println(string);
	}

	public void trace(Exception exception) {
		exception.printStackTrace();
	}
	
	public void setLocale(Locale locale) {
		resource = ResourceBundle.getBundle(sp+"lib"+sp+"resource", locale);
		tree.setLocale(locale);
		table.setLocale(locale);
		disp.setLocale(locale);
		mailgetdialog.setLocale(locale);
		mailsenddialog.setLocale(locale);
		address.setLocale(locale);
		super.setLocale(locale);
	}
	
	protected String getString(String key) {
		return (resource.getString(key));
	}
}
