/*
 * @(#)Utilities.java	0.002 99/11/16
 *
 * Copyright (c) 1999- by Daichi T.GOTO. All Rights Reserved.
 *
 */
package yuzu.util;

import java.io.*;
import java.util.*;
import yuzu.mail.*;


public class Utilities {
	private static final String separator = System.getProperty("line.separator");
	private static final String sp = System.getProperty("file.separator");
	private static String home = System.getProperty("user.home");
	private static String usrdir = System.getProperty("user.dir");
	private static File YUZU_HOME = null;
	private static Object[] YUZU_HOMES = 
	{
		new File(sp + "Program Files", ".Yuzu"),
		new File(sp + "Program Files", "Yuzu"),
		new File(home, ".Yuzu"),
		new File(home, "Yuzu"),
		new File(usrdir, ".Yuzu"),
		new File(usrdir, "Yuzu"),
	};
	private static File YUZU_CONFIGURATION_FILE;
	private static File YUZU_CONFIGURATION_DIR;
	private static File YUZU_POP3_CONFIGURATION_FILE;
	private static File YUZU_USERINFO_CONFIGURATION_FILE;
	private static File YUZU_SMTP_CONFIGURATION_FILE;
	private static File YUZU_CONTENTTYPE_CONFIGURATION_FILE;
	private static File YUZU_SIGNATURE_DIR;
	private static File YUZU_SIGNATURE_DEFAULT_DIR;
	private static File YUZU_SIGNATURE_DEFAULT_HEADER_FILE;
	private static File YUZU_SIGNATURE_DEFAULT_FOOTER_FILE;
	private static File YUZU_STATUSHOLD_FILE;
	private static File YUZU_MAIL_FILE;
	private static File YUZU_FROM_DIR;
	private static File YUZU_INBOX_DIR;
	private static File YUZU_TRASH_DIR;
	private static File YUZU_DRAFT_DIR;
	private static File YUZU_WORK_DIR;
	private static File YUZU_ADDRESSBOOK_DIR;
	
	public static final String YUZU_CACHE = ".yuzu_cache";
	public static DMail mail = new DMail();
	
	
	
	public Utilities() {
		int limit = YUZU_HOMES.length;
		for (int count=0; count<limit && YUZU_HOME == null; count++) {
			if (((File)YUZU_HOMES[count]).isDirectory() &&
				((File)YUZU_HOMES[count]).canRead() &&
				((File)YUZU_HOMES[count]).canWrite()) {
				YUZU_HOME = (File)YUZU_HOMES[count];
			}
		}
		if (YUZU_HOME == null) {
			YUZU_HOME = (File)YUZU_HOMES[2];
			YUZU_HOME.mkdirs();
		}
		YUZU_CONFIGURATION_FILE = new File(YUZU_HOME, "Configuration");
		YUZU_CONFIGURATION_DIR = new File(YUZU_HOME, "config");
		YUZU_POP3_CONFIGURATION_FILE = new File(YUZU_CONFIGURATION_DIR, "pop3");
		YUZU_USERINFO_CONFIGURATION_FILE = new File(YUZU_CONFIGURATION_DIR, "userinfo");
		YUZU_SMTP_CONFIGURATION_FILE = new File(YUZU_CONFIGURATION_DIR, "smtp");
		YUZU_CONTENTTYPE_CONFIGURATION_FILE = new File(YUZU_CONFIGURATION_DIR, "contenttype");
		YUZU_SIGNATURE_DIR = new File(YUZU_CONFIGURATION_DIR, "signature");
		YUZU_SIGNATURE_DEFAULT_DIR = new File(YUZU_SIGNATURE_DIR, "default");
		YUZU_SIGNATURE_DEFAULT_HEADER_FILE = new File(YUZU_SIGNATURE_DEFAULT_DIR, "header");
		YUZU_SIGNATURE_DEFAULT_FOOTER_FILE = new File(YUZU_SIGNATURE_DEFAULT_DIR, "footer");
		YUZU_STATUSHOLD_FILE = new File(YUZU_CONFIGURATION_DIR, "status");
		YUZU_MAIL_FILE = new File(YUZU_HOME, "Mail");
		YUZU_FROM_DIR = new File(YUZU_HOME, "from");
		YUZU_INBOX_DIR = new File(YUZU_HOME, "inbox");
		YUZU_TRASH_DIR = new File(YUZU_HOME, "trash");
		YUZU_DRAFT_DIR = new File(YUZU_HOME, "draft");
		YUZU_WORK_DIR = new File(YUZU_HOME, "work");
		YUZU_ADDRESSBOOK_DIR = new File(YUZU_HOME, "addressbook");
		if (!YUZU_CONFIGURATION_DIR.exists()) YUZU_CONFIGURATION_DIR.mkdirs();
		if (!YUZU_FROM_DIR.exists())          YUZU_FROM_DIR.mkdirs();
		if (!YUZU_INBOX_DIR.exists())         YUZU_INBOX_DIR.mkdirs();
		if (!YUZU_TRASH_DIR.exists())         YUZU_TRASH_DIR.mkdirs();
		if (!YUZU_DRAFT_DIR.exists())         YUZU_DRAFT_DIR.mkdirs();
		if (!YUZU_WORK_DIR.exists())          YUZU_WORK_DIR.mkdirs();
		if (!YUZU_ADDRESSBOOK_DIR.exists())   YUZU_ADDRESSBOOK_DIR.mkdirs();
		if (!YUZU_SIGNATURE_DIR.exists())         YUZU_SIGNATURE_DIR.mkdirs();
		if (!YUZU_SIGNATURE_DEFAULT_DIR.exists()) YUZU_SIGNATURE_DEFAULT_DIR.mkdirs();
		if (!YUZU_CONTENTTYPE_CONFIGURATION_FILE.exists()) buildContenttypeFile();

		if (!YUZU_POP3_CONFIGURATION_FILE.exists()) touch(YUZU_POP3_CONFIGURATION_FILE);
		if (!YUZU_SMTP_CONFIGURATION_FILE.exists()) touch(YUZU_SMTP_CONFIGURATION_FILE);
		if (!YUZU_USERINFO_CONFIGURATION_FILE.exists()) touch(YUZU_USERINFO_CONFIGURATION_FILE);
		if (!YUZU_SIGNATURE_DEFAULT_HEADER_FILE.exists()) touch(YUZU_SIGNATURE_DEFAULT_HEADER_FILE);
		if (!YUZU_SIGNATURE_DEFAULT_FOOTER_FILE.exists()) touch(YUZU_SIGNATURE_DEFAULT_FOOTER_FILE);
		if (!YUZU_STATUSHOLD_FILE.exists()) touch(YUZU_STATUSHOLD_FILE);
	}
	
	public File getHome() {
		return (new File(home));
	}
	
	public void setLocale(Locale locale) {
		mail.setLocale(locale);
	}

	public File getYuzuHome() {
		return(YUZU_HOME);
	}

	public File getYuzuConfigurationFile() {
		return(YUZU_CONFIGURATION_FILE);
	}
	
	public File getYuzuConfigurationDir() {
		return (YUZU_CONFIGURATION_DIR);
	}
	
	public File getYuzuPOP3ConfigurationFile() {
		return (YUZU_POP3_CONFIGURATION_FILE);
	}

	public File getYuzuUserinfoConfigurationFile() {
		return (YUZU_USERINFO_CONFIGURATION_FILE);
	}

	public File getYuzuSMTPConfigurationFile() {
		return (YUZU_SMTP_CONFIGURATION_FILE);
	}

	public File getYuzuContenttypeConfigurationFile() {
		return (YUZU_CONTENTTYPE_CONFIGURATION_FILE);
	}

	public File getYuzuMailFile() {
		return(YUZU_MAIL_FILE);
	}

	public File getFromDir() {
		return(YUZU_FROM_DIR);
	}

	public File getInboxDir() {
		return(YUZU_INBOX_DIR);
	}

	public File getTrashDir() {
		return(YUZU_TRASH_DIR);
	}

	public File getDraftDir() {
		return(YUZU_DRAFT_DIR); 
	}

	public File getWorkDir() {
		return (YUZU_WORK_DIR);
	}

	public File getAddressbookDir() {
		return (YUZU_ADDRESSBOOK_DIR);
	}

	public boolean hasDirectory(File targetDir) {
		try {
			String[] lists = targetDir.list();
			int count = lists.length - 1;
			int limit = count - 5;
			while (count > limit) {
				if ((new File(targetDir, lists[count])).isDirectory()) return (true);
				--count;
			}
		} catch(Exception exception) {
			return (false);
		}
		return (false);
	}
	
	public boolean hasAnyFileorDirectory(File targetDir) {
		try {
			String[] lists = targetDir.list();
			if (lists.length > 0) return (true);
		} catch(Exception exception) {
			return (false);
		}
		return (false);
	}

	public Vector getDirsFilesAsVector(File targetDir) {
		if (targetDir.isDirectory()) {
			String[] lists = targetDir.list();
			Vector dirs  = new Vector();
			Vector files = new Vector();
			int limit = lists.length;
			File maybeDir = null;
			for (int count=0; count<limit; count++) {
				maybeDir = new File(targetDir, lists[count]);
				if (maybeDir.isDirectory()) {
					dirs.addElement(maybeDir);
				} else {
					files.addElement(maybeDir);
				}
			}
			limit = files.size();
			for (int count=0; count<limit; count++) {
				dirs.addElement(files.elementAt(count));
			}
			return (dirs);
		}
		return (null);
	}

	public Vector getDirsAsVector(File targetDir) {
		if (targetDir.isDirectory()) {
			String[] lists = targetDir.list();
			Vector data = new Vector();
			int limit = lists.length;
			File maybeDir = null;
			for (int count=0; count<limit; count++) {
				maybeDir = new File(targetDir, lists[count]);
				if (maybeDir.isDirectory()) {
					data.addElement(maybeDir);
				}
			}
			return (data);
		}
		return (null);
	}

	public File[] getDirs(File targetDir) {
		if (targetDir.isDirectory()) {
			String[] lists = targetDir.list();
			int limit = 0;
			for (int count=0; count<lists.length; count++) {
				if ((new File(targetDir, lists[count])).isDirectory()) {
					++limit;
				}
			}
			File[] returnDirs;
			if (limit == 0) {
				returnDirs = null;
			} else {
				returnDirs = new File[limit];
				limit = 0;
				for (int count=0; count<lists.length; count++) {
					if ((new File(targetDir, lists[count])).isDirectory()) {
						returnDirs[limit] = new File(targetDir, lists[count]);
						++limit;
					}
				}
			}
			return(returnDirs);
		}
		return (null);
	}

	public File[] getFiles(File targetDir) {
		if (targetDir.isDirectory()) {
			String[] lists = targetDir.list();
			int limit = 0;
			for (int count=0; count<lists.length; count++) {
				if ((new File(targetDir, lists[count])).isFile()) {
					++limit;
				}
			}
			File[] returnFiles;
			if (limit == 0) {
				returnFiles = null;
			} else {
				returnFiles = new File[limit];
				limit = 0;
				for (int count=0; count<lists.length; count++) {
					if ((new File(targetDir, lists[count])).isFile()) {
						returnFiles[limit] = new File(targetDir, lists[count]);
						++limit;
					}
				}
			}
			return(returnFiles);
		}
		return (null);
	}

	public boolean deleteAll(File targetDir) {
		boolean returncode = true;
		String[] lists = targetDir.list();
		if (lists != null) {
			for (int count=0; count<lists.length; count++) {
				File file = new File(targetDir, lists[count]);
				if (file.isDirectory()) {
					if(!deleteAll(file)) {
						returncode = false;
						break;
					}
				} else {
					if (!file.delete()) {
						returncode = false;
						break;
					}
				}
			}
		}
		if (!targetDir.delete()) {
			returncode = false;
		}
		return(returncode);
	}

	public void buildCache(File targetDir) {
		File cache = new File(targetDir, YUZU_CACHE);
		Vector mails = getMailsAsVector(targetDir);
		BufferedWriter output = null;
		String field = null;
		File tempFile = null;
		if (mails != null) {
			try {
				output = new BufferedWriter(new FileWriter(cache));
				int limit = mails.size();
				for (int count=0; count<limit; count++) {
					tempFile = (File)(mails.elementAt(count));
					if (tempFile.length() > 0) { 
						mail.readMessage((tempFile));
						output.write(tempFile.getName());
						output.newLine();
						output.write("\tsubject: "); 
						field = mail.getMessageSubject();
						if (mail.getHeaderBool()) output.write(field);
						output.newLine();
						output.write("\tfrom: "); 
						field = mail.getMessageFrom();
						if (mail.getHeaderBool()) output.write(field);
						output.newLine();
						output.write("\tto: "); 
						field = mail.getMessageTo();
						if (mail.getHeaderBool()) output.write(field);
						output.newLine();
						output.write("\tcc: "); 
						field = mail.getMessageCc();
						if (mail.getHeaderBool()) output.write(field);
						output.newLine();
						output.write("\tdate: "); 
						field = Long.toString(mail.getMessageDateLong());
						if (mail.getHeaderBool()) output.write(field);
						output.newLine();
						output.write("\treplyto: "); 
						field = mail.getMessageReplyTo();
						if (mail.getHeaderBool()) output.write(field);
						output.newLine();
						output.write("\torganization: "); 
						field = mail.getMessageOrganization();
						if (mail.getHeaderBool()) output.write(field);
						output.newLine();
						output.write("\tmessageid: "); 
						field = mail.getMessageMessageID();
						if (mail.getHeaderBool()) output.write(field);
						output.newLine();
						output.write("\tcontenttype: "); 
						field = mail.getMessageContentType();
						if (mail.getHeaderBool()) output.write(field);
						output.newLine();
						output.write("\tmailer: "); 
						field = mail.getMessageXMailer();
						if (mail.getHeaderBool()) output.write(field);
						output.newLine();
						output.newLine();
					}
				}
				output.close();
			} catch (Exception exception) {
				try {
					output.close();
				} catch (Exception except) {
					trace(except);
				}
				trace(exception);
			}
		}
	}

	public Vector getMailsAsVector(File targetDir) {
		try {
			File[] sortedList = getMails(targetDir);
			Vector data = new Vector();
			int limit = sortedList.length;
			for (int count=0; count<limit; count++) {
				data.addElement(sortedList[count]);
			} return (data);
			
		} catch(NullPointerException exception) {
			return (null);
		}
	}

	public File[] getMails(File targetDir) {
		String[] lists = targetDir.list();
		if (lists != null) {
			int[] lists2 = new int[lists.length];
			int realcount=0;
			int temp;
			for (int count=0; count<lists.length; count++) {
				File file = new File(targetDir, lists[count]);
				if (file.isFile()) {
					try {
						temp = Integer.parseInt(lists[count]);
						lists2[realcount] = temp;
						realcount += 1;
					}
					catch (NumberFormatException err) {}
				}
			}
			if (realcount != 0) {
				int[] lists3 = new int[realcount];
				for (int count=0; count<realcount; count++) {
 					lists3[count] = lists2[count];
 				}
				boolean check=true;
				while (check) {
					check = false;
					for (int count=1; count<realcount; count++) {
						if (lists3[count-1] > lists3[count]) {
							check = true;
							int swap = lists3[count-1];
							lists3[count-1] = lists3[count];
							lists3[count] = swap;
						}
					}
				}
				File[] returnlists = new File[realcount];
				for (int count=0; count<realcount; count++) {
 					returnlists[count] = new File(targetDir, Integer.toString(lists3[count]));
 				}
				return returnlists;
			}
		}
		return (null);
	}

	public File getNextFile(File targetDir) {
		String[] lists = targetDir.list();
		File nextFile = null;
		if (lists != null) {
			int[] lists2 = new int[lists.length];
			int temp;
			int max = 0;
			for (int count=0; count<lists.length; count++) {
				File file = new File(targetDir, lists[count]);
				if (file.isFile()) {
					try {
						temp = Integer.parseInt(lists[count]);
						if (max < temp) max = temp;
					}
					catch (NumberFormatException err) {}
				}
			}
			if (max == 0) nextFile = new File(targetDir, "1");
			else          nextFile = new File(targetDir, Integer.toString(++max));
		} else {
			nextFile = new File(targetDir, "1");
		}
		return (nextFile);
	}

	public void removefromCache(File targetDir, String[] targetIndex) {
		File cache = new File(targetDir, YUZU_CACHE);
		cache.renameTo(new File(targetDir, YUZU_CACHE + ".old"));
		cache = new File(targetDir, YUZU_CACHE);
		File cache_BAK = new File(targetDir, YUZU_CACHE + ".old");
		BufferedReader input = null;
		BufferedWriter output = null;
		String line = null;
		boolean widge = false;
		try {
			input = new BufferedReader(new FileReader(cache_BAK));
			output = new BufferedWriter(new FileWriter(cache));
			line = input.readLine();
			while (line != null) {
				line = line.trim();
				if (!line.equals("")) {
					widge = false;
					for (int count=0; count<targetIndex.length && !widge; count++) {
						if (line.equals(targetIndex[count])) widge = true;
					}
					if (widge) {
						line = input.readLine();
						while (!line.trim().equals("")) {
							line = input.readLine();
						}
					} else {
						output.write(line);
						output.newLine();
						line = input.readLine();
						while (!line.trim().equals("")) {
 							output.write(line);
 							output.newLine();
							line = input.readLine();
						}
						output.newLine();
					}
				} else {
					output.newLine();
				}
				line = input.readLine();
			}
			output.flush();
			output.close();
			input.close();
			cache_BAK.delete();
		} catch(Exception exception) {
			trace(exception);
		}
	}

	public void removefromCache(File targetDir, File target) {
		removefromCache(targetDir, new String[] {target.getName()});
	}

	public void removefromCache(File targetDir, String targetIndex) {
		removefromCache(targetDir, new String[] {targetIndex});
	}

	public void addtoCache(File targetDir, File[] files) {
		try {
			RandomAccessFile cache = new RandomAccessFile(new File(targetDir, YUZU_CACHE), "rw");
			cache.seek(cache.length());
			String field = null;
			int limit = files.length;
			for (int count=0; count<limit; count++) {
				if (files[count].exists()) {
					mail.readMessage(files[count]);
					cache.write(files[count].getName().getBytes());
					cache.write(separator.getBytes());
					cache.write("\tsubject: ".getBytes()); 
					field = mail.getMessageSubject();
					if (mail.getHeaderBool()) cache.write(field.getBytes());
					cache.write(separator.getBytes());
					cache.write("\tfrom: ".getBytes()); 
					field = mail.getMessageFrom();
					if (mail.getHeaderBool()) cache.write(field.getBytes());
					cache.write(separator.getBytes());
					cache.write("\tto: ".getBytes()); 
					field = mail.getMessageTo();
					if (mail.getHeaderBool()) cache.write(field.getBytes());
					cache.write(separator.getBytes());
					cache.write("\tcc: ".getBytes()); 
					field = mail.getMessageCc();
					if (mail.getHeaderBool()) cache.write(field.getBytes());
					cache.write(separator.getBytes());
					cache.write("\tdate: ".getBytes()); 
					field = Long.toString(mail.getMessageDateLong());
					if (mail.getHeaderBool()) cache.write(field.getBytes());
					cache.write(separator.getBytes());
					cache.write("\treplyto: ".getBytes()); 
					field = mail.getMessageReplyTo();
					if (mail.getHeaderBool()) cache.write(field.getBytes());
					cache.write(separator.getBytes());
					cache.write("\torganization: ".getBytes()); 
					field = mail.getMessageOrganization();
					if (mail.getHeaderBool()) cache.write(field.getBytes());
					cache.write(separator.getBytes());
					cache.write("\tmessageid: ".getBytes()); 
					field = mail.getMessageMessageID();
					if (mail.getHeaderBool()) cache.write(field.getBytes());
					cache.write(separator.getBytes());
					cache.write("\tcontenttype: ".getBytes()); 
					field = mail.getMessageContentType();
					if (mail.getHeaderBool()) cache.write(field.getBytes());
					cache.write(separator.getBytes());
					cache.write("\tmailer: ".getBytes()); 
					field = mail.getMessageXMailer();
					if (mail.getHeaderBool()) cache.write(field.getBytes());
					cache.write(separator.getBytes());
					cache.write(separator.getBytes());
				}
			}
			cache.close();
		} catch (Exception exception) {
			trace(exception);
		}
	}

	public void addtoCache(File targetDir, File file) {
		addtoCache(targetDir, new File[] {file});
	}

	public Vector getWillSendMails() {
		Vector Mails = new Vector();
		File cache = new File(YUZU_DRAFT_DIR, YUZU_CACHE);
		BufferedReader input = null;
		String line = null;
		try {
			input = new BufferedReader(new FileReader(cache));
			line = input.readLine();
			while (line != null) {
				line = line.trim();
				try {
					Integer.parseInt(line);
					Mails.addElement(new File(YUZU_DRAFT_DIR, line));
				} catch(NumberFormatException exception) {}
				line = input.readLine();
			}
		} catch(Exception exception) {
			trace(exception);
		}
		return (Mails);
	}
	
	public Vector getPOP3Data() {
		try {
			Vector pop3Data = new Vector();
			BufferedReader input  
				= new BufferedReader(new FileReader(YUZU_POP3_CONFIGURATION_FILE));
			String linestring = input.readLine();
			String buffer = "";
			POP3Object data = null;
			while (linestring != null) {
				linestring = linestring.trim();
				if (linestring.startsWith("name:")) {
					data = new POP3Object();
					data.setName(linestring.substring(5, linestring.length()).trim());
				} else if (linestring.startsWith("server:")) {
					data.setServer(linestring.substring(7, linestring.length()).trim());
				} else if (linestring.startsWith("user:")) {
					data.setUser(linestring.substring(5, linestring.length()).trim());
				} else if (linestring.startsWith("password:")) {
					data.setPassword(linestring.substring(9, linestring.length()).trim());
				} else if (linestring.startsWith("memo:")) {
					data.setMemo(linestring.substring(5, linestring.length()).trim());
				} else if (linestring.startsWith("no_delete:")) { 
					buffer = linestring.substring(10, linestring.length()).trim();
					if (buffer.equalsIgnoreCase("true") || buffer.equalsIgnoreCase("yes")) {
						data.setNodelete(true);
					} else if (buffer.equalsIgnoreCase("false") || buffer.equalsIgnoreCase("no")) {
						data.setNodelete(false);
					}
				} else if (linestring.equals("")) {
					if (data != null) {
						pop3Data.addElement(data);
						data = null;
					}
				}
				linestring = input.readLine();
			}
			if (data != null) {
				pop3Data.addElement(data);
			}
			return (pop3Data);
		} catch (Exception exception) {
			trace(exception);
		}
		return (null);
	}

	public UserInfoObject getUserInfoData() {
		try {
			BufferedReader input  
				= new BufferedReader(new FileReader(YUZU_USERINFO_CONFIGURATION_FILE));
			String linestring = input.readLine();
			UserInfoObject data = new UserInfoObject();
			while (linestring != null) {
				linestring = linestring.trim();
				if (linestring.startsWith("username:")) {
					data.setUsername(linestring.substring(9, linestring.length()).trim());
				} else if (linestring.startsWith("fromaddress:")) {
					data.setFromaddress(linestring.substring(12, linestring.length()).trim());
				} else if (linestring.startsWith("organization:")) {
					data.setOrganization(linestring.substring(13, linestring.length()).trim());
				} 
				linestring = input.readLine();
			}
			return (data);
		} catch (Exception exception) {
			trace(exception);
		}
		return (null);
	}

	public Vector getSMTPData() {
		try {
			Vector SMTPData = new Vector();
			BufferedReader input  
				= new BufferedReader(new FileReader(YUZU_SMTP_CONFIGURATION_FILE));
			String linestring = input.readLine();
			SMTPObject data = null;
			while (linestring != null) {
				linestring = linestring.trim();
				if (linestring.startsWith("name:")) {
					data = new SMTPObject();
					data.setName(linestring.substring(5, linestring.length()).trim());
				} else if (linestring.startsWith("server:")) {
					data.setServer(linestring.substring(7, linestring.length()).trim());
				} else if (linestring.startsWith("memo:")) {
					data.setMemo(linestring.substring(5, linestring.length()).trim());
				} else if (linestring.equals("")) {
					if (data != null) {
						SMTPData.addElement(data);
						data = null;
					}
				}
				linestring = input.readLine();
			}
			if (data != null) {
				SMTPData.addElement(data);
			}
			return (SMTPData);
		} catch (Exception exception) {
			trace(exception);
		}
		return (null);
	}

	public AddressObject getAddressData(File file) {
		AddressObject data = new AddressObject();
		try {
			BufferedReader input = new BufferedReader(new FileReader(file));
			String linestring = input.readLine();
			while (linestring != null) {
				linestring = linestring.trim();
				if (linestring.startsWith("name:")) {
					data.setName(linestring.substring(5, linestring.length()).trim());
				} else if (linestring.startsWith("mailaddress:")) {
					data.setMailaddress(linestring.substring(12, linestring.length()).trim());
				} else if (linestring.startsWith("zip:")) {
					data.setZip(linestring.substring(4, linestring.length()).trim());
				} else if (linestring.startsWith("address:")) {
					data.setAddress(linestring.substring(8, linestring.length()).trim());
				} else if (linestring.startsWith("tel:")) {
					data.setTel(linestring.substring(4, linestring.length()).trim());
				} else if (linestring.startsWith("url:")) {
					data.setUrl(linestring.substring(4, linestring.length()).trim());
				} else if (linestring.startsWith("memo:")) {
					data.setMemo(linestring.substring(5, linestring.length()).trim());
				} 
				linestring = input.readLine();
			}
		} catch (Exception exception) {
			trace(exception);
		}
		return (data);
	}
	
	public SignatureObject getSignatureData() {
		SignatureObject data = new SignatureObject();
		data.setDefaultHeaderFile(YUZU_SIGNATURE_DEFAULT_HEADER_FILE);
		data.setDefaultFooterFile(YUZU_SIGNATURE_DEFAULT_FOOTER_FILE);
		return (data);
	}
	
	public SignatureObject getSignatureObject() {
		return (getSignatureData());
	}
	
	public StatusHoldObject getStatusHoldObject() {
		StatusHoldObject data = new StatusHoldObject();
		data.read(YUZU_STATUSHOLD_FILE);
		return (data);
	}

	public File getTempFile() {
		return (getNextFile(YUZU_WORK_DIR));
	}

	public boolean hasAnyMail(File targetDir) {
		if (getNextFile(targetDir).getName().equals("1")) return (false);
		return (true);
	}

	public void buildContenttypeFile() {
		try {
			String data = 
				"default: Application/Octet-Stream" + separator + 
				".ps:     Application/Postscript" + separator + 
				".pdf:    Application/Pdf" + separator + 
				".gif:    Image/Gif" + separator + 
				".jpg:    Image/Jpeg" + separator + 
				".jpeg:   Image/Jpeg" + separator + 
				".png:    Image/Png" + separator + 
				".tif:    Image/Tiff" + separator + 
				".tiff:   Image/Tiff" + separator + 
				".bmp:    Image/x-bmp" + separator + 
				".xbm:    Image/x-xbm" + separator + 
				".xpm:    Image/x-xpm" + separator + 
				".xwd:    Image/x-xwd" + separator + 
				".htm:    Text/Html" + separator + 
				".html:   Text/Html" + separator + 
				".shtml:  Text/Html" + separator + 
				".txt:    Text/Plain" + separator + 
				".jis:    Text/Plain" + separator + 
				".sjis:   Text/Plain" + separator + 
				".euc:    Text/Plain" + separator + 
				".mpg:    Video/Mpeg" + separator + 
				".mpeg:   Video/Mpeg";
			BufferedWriter output 
				= new BufferedWriter(new FileWriter(YUZU_CONTENTTYPE_CONFIGURATION_FILE));
			output.write(data);
			output.newLine(); 
			output.flush();
			output.close();
		} catch(Exception exception) {
			trace(exception);
		}
	}

	public String getOptContentType(File file) {
		return (getOptContentType(file.getPath()));
	}

	public String getOptContentType(String filename) {
		filename = filename.toLowerCase();
		String contenttype = null;
		String defaulttype = null;
		String line = null;
		try {
			BufferedReader input  
				= new BufferedReader(new FileReader(YUZU_CONTENTTYPE_CONFIGURATION_FILE));
			line = input.readLine();
			while (defaulttype == null && line != null) {
				try {
					StringTokenizer analy = new StringTokenizer(line, ":");
					if (analy.nextToken().trim().equalsIgnoreCase("default")) 
						defaulttype = analy.nextToken().trim();
				} catch(Exception exception) {
					trace(exception);
				}
				line = input.readLine();
			}
			if (defaulttype == null) defaulttype = "Application/Octet-Stream";

			input = new BufferedReader(new FileReader(YUZU_CONTENTTYPE_CONFIGURATION_FILE));
			line = input.readLine();
			String suffix = null;
			while (contenttype == null && line != null) {
				try {
					StringTokenizer analy = new StringTokenizer(line, ":");
					suffix = analy.nextToken().trim();
					suffix = suffix.toLowerCase();
					if (filename.endsWith(suffix)) contenttype = analy.nextToken().trim();
				} catch(Exception exception) {
					trace(exception);
				}
				line = input.readLine();
			}
		} catch(Exception exception) {
			trace(exception);
		}
		if (contenttype == null) contenttype = defaulttype;
		return (contenttype);
	}

	public void touch(File target) {
		try {
			BufferedWriter output = new BufferedWriter(new FileWriter(target));
			output.flush();
			output.close();
		} catch (Exception exception) {
			exception.printStackTrace();
		}
	}

	public void trace(Exception exception) {
		exception.printStackTrace();
	}

	public void print(String string) {
		System.out.println(string);
	}

	public void print(Object obj) {
		System.out.println(obj);
	}
}
